@@ -115,7 +115,7 @@ int WavpackGetMode (WavpackContext *wpc)
 // and DSD files. Except for indicating the presence of DSD data, these
 // bits are meant to simply indicate the format of the data in the original
 // source file and do NOT indicate how the library will return the data to
-// the appication (which is always the same). This means that in general an
+// the application (which is always the same). This means that in general an
 // application that simply wants to play or process the audio data need not
 // be concerned about these. If the file is DSD audio, then either of the
 // QMDOE_DSD_LSB_FIRST or QMODE_DSD_MSB_FIRST bits will be set (but the
@@ -311,7 +311,7 @@ uint32_t WavpackGetChannelLayout (WavpackContext *wpc, unsigned char *reorder)
 // This function provides the identities of ALL the channels in the file, including the
 // standard Microsoft channels (which come first, in order, and are numbered 1-18) and also
 // any non-Microsoft channels (which can be in any order and have values from 33-254). The
-// value 0x00 is invalid and 0xFF indicates an "unknown" or "unnassigned" channel. The
+// value 0x00 is invalid and 0xFF indicates an "unknown" or "unassigned" channel. The
 // string is NULL terminated so the caller must supply enough space for the number
 // of channels indicated by WavpackGetNumChannels(), plus one.
 //
@@ -617,26 +617,28 @@ void WavpackFloatNormalize (int32_t *values, int32_t num_values, int delta_exp)
 void WavpackLittleEndianToNative (void *data, char *format)
 {
     unsigned char *cp = (unsigned char *) data;
-    int64_t temp;
+    int64_t temp64;
+    int32_t temp32;
+    int16_t temp16;
 
     while (*format) {
         switch (*format) {
             case 'D':
-                temp = cp [0] + ((int64_t) cp [1] << 8) + ((int64_t) cp [2] << 16) + ((int64_t) cp [3] << 24) +
+                temp64 = cp [0] + ((int64_t) cp [1] << 8) + ((int64_t) cp [2] << 16) + ((int64_t) cp [3] << 24) +
                     ((int64_t) cp [4] << 32) + ((int64_t) cp [5] << 40) + ((int64_t) cp [6] << 48) + ((uint64_t) cp [7] << 56);
-                * (int64_t *) cp = temp;
+                memcpy (cp, &temp64, 8);
                 cp += 8;
                 break;
 
             case 'L':
-                temp = cp [0] + ((int32_t) cp [1] << 8) + ((int32_t) cp [2] << 16) + ((int64_t) cp [3] << 24);
-                * (int32_t *) cp = (int32_t) temp;
+                temp32 = cp [0] + ((int32_t) cp [1] << 8) + ((int32_t) cp [2] << 16) + ((int64_t) cp [3] << 24);
+                memcpy (cp, &temp32, 4);
                 cp += 4;
                 break;
 
             case 'S':
-                temp = cp [0] + (cp [1] << 8);
-                * (int16_t *) cp = (int16_t) temp;
+                temp16 = cp [0] + (cp [1] << 8);
+                memcpy (cp, &temp16, 2);
                 cp += 2;
                 break;
 
@@ -654,34 +656,36 @@ void WavpackLittleEndianToNative (void *data, char *format)
 void WavpackNativeToLittleEndian (void *data, char *format)
 {
     unsigned char *cp = (unsigned char *) data;
-    int64_t temp;
+    int64_t temp64;
+    int32_t temp32;
+    int16_t temp16;
 
     while (*format) {
         switch (*format) {
             case 'D':
-                temp = * (int64_t *) cp;
-                *cp++ = (unsigned char) temp;
-                *cp++ = (unsigned char) (temp >> 8);
-                *cp++ = (unsigned char) (temp >> 16);
-                *cp++ = (unsigned char) (temp >> 24);
-                *cp++ = (unsigned char) (temp >> 32);
-                *cp++ = (unsigned char) (temp >> 40);
-                *cp++ = (unsigned char) (temp >> 48);
-                *cp++ = (unsigned char) (temp >> 56);
+                memcpy (&temp64, cp, sizeof (temp64));
+                *cp++ = (unsigned char) temp64;
+                *cp++ = (unsigned char) (temp64 >> 8);
+                *cp++ = (unsigned char) (temp64 >> 16);
+                *cp++ = (unsigned char) (temp64 >> 24);
+                *cp++ = (unsigned char) (temp64 >> 32);
+                *cp++ = (unsigned char) (temp64 >> 40);
+                *cp++ = (unsigned char) (temp64 >> 48);
+                *cp++ = (unsigned char) (temp64 >> 56);
                 break;
 
             case 'L':
-                temp = * (int32_t *) cp;
-                *cp++ = (unsigned char) temp;
-                *cp++ = (unsigned char) (temp >> 8);
-                *cp++ = (unsigned char) (temp >> 16);
-                *cp++ = (unsigned char) (temp >> 24);
+                memcpy (&temp32, cp, sizeof (temp32));
+                *cp++ = (unsigned char) temp32;
+                *cp++ = (unsigned char) (temp32 >> 8);
+                *cp++ = (unsigned char) (temp32 >> 16);
+                *cp++ = (unsigned char) (temp32 >> 24);
                 break;
 
             case 'S':
-                temp = * (int16_t *) cp;
-                *cp++ = (unsigned char) temp;
-                *cp++ = (unsigned char) (temp >> 8);
+                memcpy (&temp16, cp, sizeof (temp16));
+                *cp++ = (unsigned char) temp16;
+                *cp++ = (unsigned char) (temp16 >> 8);
                 break;
 
             default:
@@ -698,26 +702,28 @@ void WavpackNativeToLittleEndian (void *data, char *format)
 void WavpackBigEndianToNative (void *data, char *format)
 {
     unsigned char *cp = (unsigned char *) data;
-    int64_t temp;
+    int64_t temp64;
+    int32_t temp32;
+    int16_t temp16;
 
     while (*format) {
         switch (*format) {
             case 'D':
-                temp = cp [7] + ((int64_t) cp [6] << 8) + ((int64_t) cp [5] << 16) + ((int64_t) cp [4] << 24) +
+                temp64 = cp [7] + ((int64_t) cp [6] << 8) + ((int64_t) cp [5] << 16) + ((int64_t) cp [4] << 24) +
                     ((int64_t) cp [3] << 32) + ((int64_t) cp [2] << 40) + ((int64_t) cp [1] << 48) + ((uint64_t) cp [0] << 56);
-                * (int64_t *) cp = temp;
+                memcpy (cp, &temp64, 8);
                 cp += 8;
                 break;
 
             case 'L':
-                temp = cp [3] + ((int32_t) cp [2] << 8) + ((int32_t) cp [1] << 16) + ((int64_t) cp [0] << 24);
-                * (int32_t *) cp = (int32_t) temp;
+                temp32 = cp [3] + ((int32_t) cp [2] << 8) + ((int32_t) cp [1] << 16) + ((int64_t) cp [0] << 24);
+                memcpy (cp, &temp32, 4);
                 cp += 4;
                 break;
 
             case 'S':
-                temp = cp [1] + (cp [0] << 8);
-                * (int16_t *) cp = (int16_t) temp;
+                temp16 = cp [1] + (cp [0] << 8);
+                memcpy (cp, &temp16, 2);
                 cp += 2;
                 break;
 
@@ -735,34 +741,36 @@ void WavpackBigEndianToNative (void *data, char *format)
 void WavpackNativeToBigEndian (void *data, char *format)
 {
     unsigned char *cp = (unsigned char *) data;
-    int64_t temp;
+    int64_t temp64;
+    int32_t temp32;
+    int16_t temp16;
 
     while (*format) {
         switch (*format) {
             case 'D':
-                temp = * (int64_t *) cp;
-                *cp++ = (unsigned char) (temp >> 56);
-                *cp++ = (unsigned char) (temp >> 48);
-                *cp++ = (unsigned char) (temp >> 40);
-                *cp++ = (unsigned char) (temp >> 32);
-                *cp++ = (unsigned char) (temp >> 24);
-                *cp++ = (unsigned char) (temp >> 16);
-                *cp++ = (unsigned char) (temp >> 8);
-                *cp++ = (unsigned char) temp;
+                memcpy (&temp64, cp, sizeof (temp64));
+                *cp++ = (unsigned char) (temp64 >> 56);
+                *cp++ = (unsigned char) (temp64 >> 48);
+                *cp++ = (unsigned char) (temp64 >> 40);
+                *cp++ = (unsigned char) (temp64 >> 32);
+                *cp++ = (unsigned char) (temp64 >> 24);
+                *cp++ = (unsigned char) (temp64 >> 16);
+                *cp++ = (unsigned char) (temp64 >> 8);
+                *cp++ = (unsigned char) temp64;
                 break;
 
             case 'L':
-                temp = * (int32_t *) cp;
-                *cp++ = (unsigned char) (temp >> 24);
-                *cp++ = (unsigned char) (temp >> 16);
-                *cp++ = (unsigned char) (temp >> 8);
-                *cp++ = (unsigned char) temp;
+                memcpy (&temp32, cp, sizeof (temp32));
+                *cp++ = (unsigned char) (temp32 >> 24);
+                *cp++ = (unsigned char) (temp32 >> 16);
+                *cp++ = (unsigned char) (temp32 >> 8);
+                *cp++ = (unsigned char) temp32;
                 break;
 
             case 'S':
-                temp = * (int16_t *) cp;
-                *cp++ = (unsigned char) (temp >> 8);
-                *cp++ = (unsigned char) temp;
+                memcpy (&temp16, cp, sizeof (temp16));
+                *cp++ = (unsigned char) (temp16 >> 8);
+                *cp++ = (unsigned char) temp16;
                 break;
 
             default:
