@@ -18,6 +18,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "wavpack_local.h"
 
@@ -358,7 +359,7 @@ int unpack_init (WavpackContext *wpc)
     return TRUE;
 }
 
-//////////////////////////////// matadata handlers ///////////////////////////////
+//////////////////////////////// metadata handlers ///////////////////////////////
 
 // These functions handle specific metadata types and are called directly
 // during WavPack block parsing by process_metadata() at the bottom.
@@ -796,8 +797,13 @@ static int process_metadata (WavpackContext *wpc, WavpackMetadata *wpmd)
 
         case ID_ALT_EXTENSION:
             if (wpmd->byte_length && wpmd->byte_length < sizeof (wpc->file_extension)) {
-                memcpy (wpc->file_extension, wpmd->data, wpmd->byte_length);
-                wpc->file_extension [wpmd->byte_length] = 0;
+                int i, j;
+
+                for (i = j = 0; i < wpmd->byte_length; ++i)
+                    if (isalnum (((char *) wpmd->data) [i]))
+                        wpc->file_extension [j++] = ((char *) wpmd->data) [i];
+
+                wpc->file_extension [j] = 0;
             }
 
             return TRUE;
@@ -829,7 +835,7 @@ static void bs_open_read (Bitstream *bs, void *buffer_start, void *buffer_end)
 }
 
 // This function is only called from the getbit() and getbits() macros when
-// the BitStream has been exhausted and more data is required. Sinve these
+// the BitStream has been exhausted and more data is required. Since these
 // bistreams no longer access files, this function simple sets an error and
 // resets the buffer.
 
