@@ -17,11 +17,13 @@
 
 #include "wavpack_local.h"
 
+#ifndef __WATCOMC__
 #ifdef _WIN32
 #define stricmp(x,y) _stricmp(x,y)
 #else
 #define stricmp strcasecmp
 #endif
+#endif
 
 static int get_ape_tag_item (M_Tag *m_tag, const char *item, char *value, int size, int type);
 static int get_id3_tag_item (M_Tag *m_tag, const char *item, char *value, int size);
@@ -523,7 +525,7 @@ static int write_tag_reader (WavpackContext *wpc)
 
     if (result && tag_size < -m_tag->tag_file_pos && !wpc->reader->truncate_here) {
         int nullcnt = (int) (-m_tag->tag_file_pos - tag_size);
-        char zero [1] = { 0 };
+        char zero = 0;
 
         while (nullcnt--)
             wpc->reader->write_bytes (wpc->wv_in, &zero, 1);
