@@ -36,7 +36,11 @@ int init_dsd_block (WavpackContext *wpc, WavpackMetadata *wpmd)
 
     wps->dsd.byteptr = (unsigned char *)wpmd->data;
     wps->dsd.endptr = wps->dsd.byteptr + wpmd->byte_length;
-    wpc->dsd_multiplier = 1 << *wps->dsd.byteptr++;
+
+    if (*wps->dsd.byteptr > 31)
+        return FALSE;
+
+    wpc->dsd_multiplier = 1U << *wps->dsd.byteptr++;
     wps->dsd.mode = *wps->dsd.byteptr++;
 
     if (!wps->dsd.mode) {
@@ -144,7 +148,7 @@ static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd)
     lb_ptr = wps->dsd.lookup_buffer = (unsigned char *)malloc (wps->dsd.history_bins * MAX_BYTES_PER_BIN);
     wps->dsd.value_lookup = (unsigned char **)malloc (sizeof (*wps->dsd.value_lookup) * wps->dsd.history_bins);
     memset (wps->dsd.value_lookup, 0, sizeof (*wps->dsd.value_lookup) * wps->dsd.history_bins);
-    wps->dsd.summed_probabilities = (int16_t (*)[256])malloc (sizeof (*wps->dsd.summed_probabilities) * wps->dsd.history_bins);
+    wps->dsd.summed_probabilities = (uint16_t (*)[256])malloc (sizeof (*wps->dsd.summed_probabilities) * wps->dsd.history_bins);
     wps->dsd.probabilities = (unsigned char (*)[256])malloc (sizeof (*wps->dsd.probabilities) * wps->dsd.history_bins);
 
     max_probability = *wps->dsd.byteptr++;
@@ -220,7 +224,7 @@ static int decode_fast (WavpackStream *wps, int32_t *output, int sample_count)
         total_samples *= 2;
 
     while (total_samples--) {
-        int mult, index, code, i;
+        unsigned int mult, index, code, i;
 
         if (!wps->dsd.summed_probabilities [wps->dsd.p0] [255])
             return 0;
@@ -334,7 +338,7 @@ static int init_dsd_block_high (WavpackStream *wps, WavpackMetadata *wpmd)
         sp->filter6 = 0;
         sp->factor = *wps->dsd.byteptr++ & 0xff;
         sp->factor |= (*wps->dsd.byteptr++ << 8) & 0xff00;
-        sp->factor = (sp->factor << 16) >> 16;
+        sp->factor = (int32_t)((uint32_t)sp->factor << 16) >> 16;
     }
 
     wps->dsd.high = 0xffffffff;
@@ -382,9 +386,9 @@ static int decode_high (WavpackStream *wps, int32_t *output, int sample_count)
                 wps->dsd.low <<= 8;
             }
 
-            sp [0].value += sp [0].filter6 << 3;
+            sp [0].value += sp [0].filter6 * 8;
             sp [0].byte = (sp [0].byte << 1) | (sp [0].filter0 & 1);
-            sp [0].factor += (((sp [0].value ^ sp [0].filter0) >> 31) | 1) & ((sp [0].value ^ (sp [0].value - (sp [0].filter6 << 4))) >> 31);
+            sp [0].factor += (((sp [0].value ^ sp [0].filter0) >> 31) | 1) & ((sp [0].value ^ (sp [0].value - (sp [0].filter6 * 16))) >> 31);
             sp [0].filter1 += ((sp [0].filter0 & VALUE_ONE) - sp [0].filter1) >> 6;
             sp [0].filter2 += ((sp [0].filter0 & VALUE_ONE) - sp [0].filter2) >> 4;
             sp [0].filter3 += (sp [0].filter2 - sp [0].filter3) >> 4;
@@ -417,9 +421,9 @@ static int decode_high (WavpackStream *wps, int32_t *output, int sample_count)
                 wps->dsd.low <<= 8;
             }
 
-            sp [1].value += sp [1].filter6 << 3;
+            sp [1].value += sp [1].filter6 * 8;
             sp [1].byte = (sp [1].byte << 1) | (sp [1].filter0 & 1);
-            sp [1].factor += (((sp [1].value ^ sp [1].filter0) >> 31) | 1) & ((sp [1].value ^ (sp [1].value - (sp [1].filter6 << 4))) >> 31);
+            sp [1].factor += (((sp [1].value ^ sp [1].filter0) >> 31) | 1) & ((sp [1].value ^ (sp [1].value - (sp [1].filter6 * 16))) >> 31);
             sp [1].filter1 += ((sp [1].filter0 & VALUE_ONE) - sp [1].filter1) >> 6;
             sp [1].filter2 += ((sp [1].filter0 & VALUE_ONE) - sp [1].filter2) >> 4;
             sp [1].filter3 += (sp [1].filter2 - sp [1].filter3) >> 4;
@@ -494,9 +498,11 @@ typedef struct {
 typedef struct {
     int32_t conv_tables [HISTORY_BYTES] [256];
     DecimationChannel *chans;
-    int num_channels;
+    int num_channels, reset;
 } DecimationContext;
 
+static void extrapolate_pcm (int32_t *samples, int samples_to_extrapolate, int samples_visible, int num_channels);
+
 void *decimate_dsd_init (int num_channels)
 {
     DecimationContext *context = (DecimationContext *)malloc (sizeof (DecimationContext));
@@ -553,19 +559,22 @@ void decimate_dsd_reset (void *decimate_context)
     for (chan = 0; chan < context->num_channels; ++chan)
         for (i = 0; i < HISTORY_BYTES; ++i)
             context->chans [chan].delay [i] = 0x55;
+
+    context->reset = 1;
 }
 
 void decimate_dsd_run (void *decimate_context, int32_t *samples, int num_samples)
 {
     DecimationContext *context = (DecimationContext *) decimate_context;
-    int chan = 0;
+    int chan = 0, scount = num_samples;
+    int32_t *samptr = samples;
 
     if (!context)
         return;
 
-    while (num_samples) {
+    while (scount) {
         DecimationChannel *sp = context->chans + chan;
-        int sum = 0;
+        int32_t sum = 0;
 
 #if (HISTORY_BYTES == 10)
         sum += context->conv_tables [0] [sp->delay [0] = sp->delay [1]];
@@ -577,7 +586,7 @@ void decimate_dsd_run (void *decimate_context, int32_t *samples, int num_samples
         sum += context->conv_tables [6] [sp->delay [6] = sp->delay [7]];
         sum += context->conv_tables [7] [sp->delay [7] = sp->delay [8]];
         sum += context->conv_tables [8] [sp->delay [8] = sp->delay [9]];
-        sum += context->conv_tables [9] [sp->delay [9] = *samples];
+        sum += context->conv_tables [9] [sp->delay [9] = *samptr];
 #elif (HISTORY_BYTES == 7)
         sum += context->conv_tables [0] [sp->delay [0] = sp->delay [1]];
         sum += context->conv_tables [1] [sp->delay [1] = sp->delay [2]];
@@ -585,23 +594,75 @@ void decimate_dsd_run (void *decimate_context, int32_t *samples, int num_samples
         sum += context->conv_tables [3] [sp->delay [3] = sp->delay [4]];
         sum += context->conv_tables [4] [sp->delay [4] = sp->delay [5]];
         sum += context->conv_tables [5] [sp->delay [5] = sp->delay [6]];
-        sum += context->conv_tables [6] [sp->delay [6] = *samples];
+        sum += context->conv_tables [6] [sp->delay [6] = *samptr];
 #else
         int i;
 
         for (i = 0; i < HISTORY_BYTES-1; ++i)
             sum += context->conv_tables [i] [sp->delay [i] = sp->delay [i+1]];
 
-        sum += context->conv_tables [i] [sp->delay [i] = *samples];
+        sum += context->conv_tables [i] [sp->delay [i] = *samptr];
 #endif
 
-        *samples++ = sum >> 4;
+        *samptr++ = (sum + 8) >> 4;
 
         if (++chan == context->num_channels) {
-            num_samples--;
+            scount--;
             chan = 0;
         }
     }
+
+    if (context->reset) {
+        extrapolate_pcm (samples, HISTORY_BYTES - 1, num_samples, context->num_channels);
+        context->reset = 0;
+    }
+}
+
+// This function is used to linearly extrapolate some samples at the beginning of the first
+// decoded frame because we don't have the previous DSD data to prefill the decimation filter.
+// Currently we only extrapolate at the beginning of the file because we have an implicit
+// delay in the decimation. It might be better, but more complicated, to have zero delay in
+// the decimation and split the extrapolated samples between the beginning and end of the
+// file.
+
+static void extrapolate_pcm (int32_t *samples, int samples_to_extrapolate, int samples_visible, int num_channels)
+{
+    int scount = num_channels, min_period = 5, max_period = 10;
+
+    if (samples_visible < samples_to_extrapolate + min_period * 2)
+        return;
+
+    if (samples_visible < samples_to_extrapolate + max_period * 2)
+        max_period = (samples_visible - samples_to_extrapolate) / 2;
+
+    while (scount--) {
+        float left_value_ave = 0.0, right_value_ave = 0.0, slope;
+        int period, i;
+
+        for (period = min_period; period <= max_period; ++period) {
+            float left_ratio = (samples_to_extrapolate + period / 2.0F) / period, right_ratio = (period / 2.0F) / period;
+            int32_t *sam1 = samples + samples_to_extrapolate * num_channels, *sam2 = sam1 + period * num_channels;
+            float ave1 = 0.0, ave2 = 0.0;
+            int i;
+
+            for (i = 0; i < period; ++i) {
+                ave1 += (float) sam1 [i * num_channels] / period;
+                ave2 += (float) sam2 [i * num_channels] / period;
+            }
+
+            left_value_ave += ave1 + (ave1 - ave2) * left_ratio;
+            right_value_ave += ave1 + (ave1 - ave2) * right_ratio;
+        }
+
+        right_value_ave /= (max_period - min_period + 1);
+        left_value_ave /= (max_period - min_period + 1);
+        slope = (right_value_ave - left_value_ave) / (samples_to_extrapolate - 1);
+
+        for (i = 0; i < samples_to_extrapolate; ++i)
+            samples [i * num_channels] = (int32_t) (left_value_ave + i * slope + 0.5);
+
+        samples++;
+    }
 }
 
 void decimate_dsd_destroy (void *decimate_context)
