@@ -13,14 +13,26 @@
 
 #include "wavpack.h"
 
-#if defined(_WIN32)
+#if defined(_WIN32) || (defined(__WATCOMC__) && defined(__OS2__))
 #define strdup(x) _strdup(x)
+#ifndef FASTCALL
+#if defined(_M_IX86) || defined(__i386__)
 #define FASTCALL __fastcall
+#else /* !x86: */
+#define FASTCALL
+#endif
+#endif /* FASTCALL */
 #else
 #define FASTCALL
 #endif
 
-#if defined(_WIN32) || \
+#if defined(__WATCOMC__) && defined(OPT_ASM_X86)
+#define ASMCALL __cdecl
+#else
+#define ASMCALL
+#endif
+
+#if defined(_WIN32) || defined(__WATCOMC__) || \
     (defined(BYTE_ORDER) && defined(LITTLE_ENDIAN) && (BYTE_ORDER == LITTLE_ENDIAN)) || \
     (defined(__BYTE_ORDER__) && defined(__ORDER_LITTLE_ENDIAN__) && (__BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__))
 #define BITSTREAM_SHORTS    // use 16-bit "shorts" for reading/writing bitstreams (instead of chars)
@@ -62,8 +74,8 @@ typedef int32_t f32;
 #define get_sign(f)         (((f) >> 31) & 0x1)
 
 #define set_mantissa(f,v)   (f) ^= (((f) ^ (v)) & 0x7fffff)
-#define set_exponent(f,v)   (f) ^= (((f) ^ ((v) << 23)) & 0x7f800000)
-#define set_sign(f,v)       (f) ^= (((f) ^ ((v) << 31)) & 0x80000000)
+#define set_exponent(f,v)   (f) ^= (((f) ^ ((uint32_t)(v) << 23)) & 0x7f800000)
+#define set_sign(f,v)       (f) ^= (((f) ^ ((uint32_t)(v) << 31)) & 0x80000000)
 
 #include <stdio.h>
 
@@ -114,41 +126,15 @@ typedef struct {
     unsigned char id;
 } WavpackMetadata;
 
-#define ID_UNIQUE               0x3f
-#define ID_OPTIONAL_DATA        0x20
-#define ID_ODD_SIZE             0x40
-#define ID_LARGE                0x80
 
-#define ID_DUMMY                0x0
-#define ID_ENCODER_INFO         0x1
-#define ID_DECORR_TERMS         0x2
-#define ID_DECORR_WEIGHTS       0x3
-#define ID_DECORR_SAMPLES       0x4
-#define ID_ENTROPY_VARS         0x5
-#define ID_HYBRID_PROFILE       0x6
-#define ID_SHAPING_WEIGHTS      0x7
-#define ID_FLOAT_INFO           0x8
-#define ID_INT32_INFO           0x9
-#define ID_WV_BITSTREAM         0xa
-#define ID_WVC_BITSTREAM        0xb
-#define ID_WVX_BITSTREAM        0xc
-#define ID_CHANNEL_INFO         0xd
-#define ID_DSD_BLOCK            0xe
+///////////////////////// WavPack Configuration ///////////////////////////////
 
-#define ID_RIFF_HEADER          (ID_OPTIONAL_DATA | 0x1)
-#define ID_RIFF_TRAILER         (ID_OPTIONAL_DATA | 0x2)
-#define ID_ALT_HEADER           (ID_OPTIONAL_DATA | 0x3)
-#define ID_ALT_TRAILER          (ID_OPTIONAL_DATA | 0x4)
-#define ID_CONFIG_BLOCK         (ID_OPTIONAL_DATA | 0x5)
-#define ID_MD5_CHECKSUM         (ID_OPTIONAL_DATA | 0x6)
-#define ID_SAMPLE_RATE          (ID_OPTIONAL_DATA | 0x7)
-#define ID_ALT_EXTENSION        (ID_OPTIONAL_DATA | 0x8)
-#define ID_ALT_MD5_CHECKSUM     (ID_OPTIONAL_DATA | 0x9)
-#define ID_NEW_CONFIG_BLOCK     (ID_OPTIONAL_DATA | 0xa)
-#define ID_CHANNEL_IDENTITIES   (ID_OPTIONAL_DATA | 0xb)
-#define ID_BLOCK_CHECKSUM       (ID_OPTIONAL_DATA | 0xf)
+/*
+ * These config flags are not actually used for external configuration, which is
+ * why they're not in the external wavpack.h file, but they are used internally
+ * in the flags field of the WavpackConfig struct.
+ */
 
-#define CONFIG_BYTES_STORED     3       // 1-4 bytes/sample
 #define CONFIG_MONO_FLAG        4       // not stereo
 #define CONFIG_FLOAT_DATA       0x80    // ieee 32-bit floating point data
 
@@ -244,7 +230,8 @@ struct words_data {
 };
 
 typedef struct {
-    int32_t value, filter0, filter1, filter2, filter3, filter4, filter5, filter6, factor, byte;
+    int32_t value, filter0, filter1, filter2, filter3, filter4, filter5, filter6, factor;
+    unsigned int byte;
 } DSDfilters;
 
 typedef struct {
@@ -279,7 +266,7 @@ typedef struct {
     struct {
         unsigned char *byteptr, *endptr, (*probabilities) [256], *lookup_buffer, **value_lookup, mode, ready;
         int history_bins, p0, p1;
-        int16_t (*summed_probabilities) [256];
+        uint16_t (*summed_probabilities) [256];
         uint32_t low, high, value;
         DSDfilters filters [2];
         int32_t *ptable;
@@ -342,6 +329,7 @@ struct WavpackContext {
 //////////////////////// function prototypes and macros //////////////////////
 
 #define CLEAR(destin) memset (&destin, 0, sizeof (destin));
+#define CLEARA(destin) memset (destin, 0, sizeof (destin)); /* for arrays */
 
 //////////////////////////////// decorrelation //////////////////////////////
 // modules: pack.c, unpack.c, unpack_floats.c, extra1.c, extra2.c
@@ -350,7 +338,7 @@ struct WavpackContext {
 
 // These macros implement the weight application and update operations
 // that are at the heart of the decorrelation loops. Note that there are
-// sometimes two and even three versions of each macro. Theses should be
+// sometimes two and even three versions of each macro. These should be
 // equivalent and produce identical results, but some may perform better
 // or worse on a given architecture.
 
@@ -429,7 +417,8 @@ void decimate_dsd_destroy (void *decimate_context);
 
 ///////////////////////////////// CPU feature detection ////////////////////////////////
 
-int unpack_cpu_has_feature_x86 (int findex), pack_cpu_has_feature_x86 (int findex);
+int ASMCALL unpack_cpu_has_feature_x86 (int findex);
+int ASMCALL pack_cpu_has_feature_x86 (int findex);
 
 #define CPU_FEATURE_MMX     23
 
@@ -460,7 +449,7 @@ uint32_t bs_close_read (Bitstream *bs);
 #define getbits(value, nbits, bs) do { \
     while ((nbits) > (bs)->bc) { \
         if (++((bs)->ptr) == (bs)->end) (bs)->wrap (bs); \
-        (bs)->sr |= (int32_t)*((bs)->ptr) << (bs)->bc; \
+        (bs)->sr |= (uint32_t)*((bs)->ptr) << (bs)->bc; \
         (bs)->bc += sizeof (*((bs)->ptr)) * 8; \
     } \
     *(value) = (bs)->sr; \
@@ -474,7 +463,7 @@ uint32_t bs_close_read (Bitstream *bs);
     } \
 } while (0)
 
-#define putbit(bit, bs) do { if (bit) (bs)->sr |= (1 << (bs)->bc); \
+#define putbit(bit, bs) do { if (bit) (bs)->sr |= (1U << (bs)->bc); \
     if (++((bs)->bc) == sizeof (*((bs)->ptr)) * 8) { \
         *((bs)->ptr) = (bs)->sr; \
         (bs)->sr = (bs)->bc = 0; \
@@ -488,7 +477,7 @@ uint32_t bs_close_read (Bitstream *bs);
         if (++((bs)->ptr) == (bs)->end) (bs)->wrap (bs); \
     }} while (0)
 
-#define putbit_1(bs) do { (bs)->sr |= (1 << (bs)->bc); \
+#define putbit_1(bs) do { (bs)->sr |= (1U << (bs)->bc); \
     if (++((bs)->bc) == sizeof (*((bs)->ptr)) * 8) { \
         *((bs)->ptr) = (bs)->sr; \
         (bs)->sr = (bs)->bc = 0; \
@@ -496,7 +485,7 @@ uint32_t bs_close_read (Bitstream *bs);
     }} while (0)
 
 #define putbits(value, nbits, bs) do { \
-    (bs)->sr |= (int32_t)(value) << (bs)->bc; \
+    (bs)->sr |= (uint32_t)(value) << (bs)->bc; \
     if (((bs)->bc += (nbits)) >= sizeof (*((bs)->ptr)) * 8) \
         do { \
             *((bs)->ptr) = (bs)->sr; \
@@ -539,6 +528,14 @@ uint32_t bs_close_read (Bitstream *bs);
 
 #ifdef HAVE___BUILTIN_CLZ
 #define count_bits(av) ((av) ? 32 - __builtin_clz (av) : 0)
+#elif defined (__WATCOMC__) && defined(__386__)
+extern __inline int _bsr_watcom(uint32_t);
+#pragma aux _bsr_watcom = \
+  "bsr eax, eax" \
+  parm [eax] nomemory \
+  value [eax] \
+  modify exact [eax] nomemory;
+#define count_bits(av) ((av) ? _bsr_watcom((av)) + 1 : 0)
 #elif defined (_WIN64)
 static __inline int count_bits (uint32_t av) { unsigned long res; return _BitScanReverse (&res, av) ? (int)(res + 1) : 0; }
 #else
@@ -583,12 +580,12 @@ int FASTCALL wp_log2 (uint32_t avalue);
 #define LOG2BUFFER log2buffer
 #endif
 
-uint32_t LOG2BUFFER (int32_t *samples, uint32_t num_samples, int limit);
+uint32_t ASMCALL LOG2BUFFER (int32_t *samples, uint32_t num_samples, int limit);
 
 signed char store_weight (int weight);
 int restore_weight (signed char weight);
 
-#define WORD_EOF ((int32_t)(1L << 31))
+#define WORD_EOF ((int32_t)(1U << 31))
 
 void WavpackFloatNormalize (int32_t *values, int32_t num_values, int delta_exp);
 
